@@ -6,16 +6,14 @@ from container_app_conf.formatter.toml import TomlFormatter
 from requests import HTTPError
 from telegram import Update, ParseMode, InlineKeyboardMarkup, InlineKeyboardButton, ReplyKeyboardRemove
 from telegram.ext import CommandHandler, Filters, MessageHandler, Updater, CallbackContext, CallbackQueryHandler
-from telegram_click.argument import Argument
+from telegram_click.argument import Argument, Flag
 from telegram_click.decorator import command
 
 from keel_telegram_bot.api_client import KeelApiClient
 from keel_telegram_bot.bot.permissions import CONFIG_ADMINS
 from keel_telegram_bot.bot.reply_keyboard_handler import ReplyKeyboardHandler
 from keel_telegram_bot.config import Config
-from keel_telegram_bot.const import *
-from keel_telegram_bot.stats import COMMAND_TIME_START, COMMAND_TIME_LIST_APPROVALS, COMMAND_TIME_APPROVE, \
-    COMMAND_TIME_REJECT, COMMAND_TIME_DELETE
+from keel_telegram_bot.stats import *
 from keel_telegram_bot.util import send_message, approval_to_str
 
 LOGGER = logging.getLogger(__name__)
@@ -123,12 +121,13 @@ class KeelTelegramBot:
     @command(name=COMMAND_LIST_APPROVALS,
              description="List pending approvals",
              arguments=[
-                 Argument(name=["rejected", "r"], description="Rejected", type=bool, example="true", optional=True),
-                 Argument(name=["archived", "a"], description="Archived", type=bool, example="true", optional=True),
+                 Flag(name=["archived", "h"], description="Include archived items"),
+                 Flag(name=["approved", "a"], description="Include approved items"),
+                 Flag(name=["rejected", "r"], description="Include rejected items"),
              ],
              permissions=CONFIG_ADMINS)
     def _list_approvals_callback(self, update: Update, context: CallbackContext,
-                                 rejected: bool or None, archived: bool or None) -> None:
+                                 archived: bool, approved: bool, rejected: bool) -> None:
         """
         List pending approvals
         """
@@ -136,7 +135,7 @@ class KeelTelegramBot:
         message = update.effective_message
         chat_id = update.effective_chat.id
 
-        items = self._api_client.get_approvals(rejected, archived)
+        items = self._api_client.get_approvals()
 
         rejected_items = list(filter(lambda x: x[KEY_REJECTED], items))
         archived_items = list(filter(lambda x: x[KEY_ARCHIVED], items))
@@ -146,24 +145,42 @@ class KeelTelegramBot:
         approved_items = list(
             filter(lambda x: x not in rejected_items and x not in archived_items and x not in pending_items, items))
 
-        text = "\n".join([
-            f"<b>Pending ({len(pending_items)}):</b>",
-            "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), pending_items))),
-            f"<b>Approved ({len(approved_items)}):</b>",
-            "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), approved_items))),
-            f"<b>Rejected ({len(rejected_items)}):</b>",
-            "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), rejected_items))),
-            f"<b>Archived ({len(archived_items)}):</b>",
-            "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), archived_items)))
-        ])
+        lines = []
+        if archived:
+            lines.append("\n".join([
+                f"<b>=== Archived ({len(archived_items)}) ===</b>",
+                "",
+                "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), archived_items)))
+            ]).strip())
 
+        if approved:
+            lines.append("\n".join([
+                f"<b>=== Approved ({len(approved_items)}) ===</b>",
+                "",
+                "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), approved_items))),
+            ]).strip())
+
+        if rejected:
+            lines.append("\n".join([
+                f"<b>=== Rejected ({len(rejected_items)}) ===</b>",
+                "",
+                "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), rejected_items))),
+            ]).strip())
+
+        lines.append("\n".join([
+            f"<b>=== Pending ({len(pending_items)}) ===</b>",
+            "",
+            "\n\n".join(list(map(lambda x: "> " + approval_to_str(x), pending_items))),
+        ]))
+
+        text = "\n\n".join(lines).strip()
         send_message(bot, chat_id, text, reply_to=message.message_id, parse_mode=ParseMode.HTML)
 
     @COMMAND_TIME_APPROVE.time()
     @command(name=COMMAND_APPROVE,
              description="Approve a pending item",
              arguments=[
-                 Argument(name=["identifier", "i"], description="Approval identifier",
+                 Argument(name=["identifier", "i"], description="Approval identifier or id",
                           example="default/myimage:1.5.5"),
                  Argument(name=["voter", "v"], description="Name of voter", example="john", optional=True),
              ],
@@ -186,6 +203,12 @@ class KeelTelegramBot:
             send_message(bot, chat_id, text, reply_to=message.message_id, menu=ReplyKeyboardRemove(selective=True))
 
         items = self._api_client.get_approvals(rejected=False, archived=False)
+
+        exact_matches = list(filter(lambda x: x["id"] == identifier, items))
+        if len(exact_matches) > 0:
+            execute(update, context, exact_matches[0], {})
+            return
+
         self._response_handler.await_user_selection(
             update, context, identifier, choices=items, key=lambda x: x["identifier"],
             callback=execute,
@@ -195,7 +218,8 @@ class KeelTelegramBot:
     @command(name=COMMAND_REJECT,
              description="Reject a pending item",
              arguments=[
-                 Argument(name=["identifier", "i"], description="Approval identifier", example="default/myimage:1.5.5"),
+                 Argument(name=["identifier", "i"], description="Approval identifier or id",
+                          example="default/myimage:1.5.5"),
                  Argument(name=["voter", "v"], description="Name of voter", example="john", optional=True),
              ],
              permissions=CONFIG_ADMINS)
@@ -217,6 +241,12 @@ class KeelTelegramBot:
             send_message(bot, chat_id, text, reply_to=message.message_id, menu=ReplyKeyboardRemove(selective=True))
 
         items = self._api_client.get_approvals(rejected=False, archived=False)
+
+        exact_matches = list(filter(lambda x: x["id"] == identifier, items))
+        if len(exact_matches) > 0:
+            execute(update, context, exact_matches[0], {})
+            return
+
         self._response_handler.await_user_selection(
             update, context, identifier, choices=items, key=lambda x: x["identifier"],
             callback=execute,
@@ -226,7 +256,8 @@ class KeelTelegramBot:
     @command(name=COMMAND_DELETE,
              description="Delete an approval item",
              arguments=[
-                 Argument(name=["identifier", "i"], description="Approval identifier", example="default/myimage:1.5.5"),
+                 Argument(name=["identifier", "i"], description="Approval identifier or id",
+                          example="default/myimage:1.5.5"),
                  Argument(name=["voter", "v"], description="Name of voter", example="john", optional=True),
              ],
              permissions=CONFIG_ADMINS)
@@ -248,6 +279,12 @@ class KeelTelegramBot:
             send_message(bot, chat_id, text, reply_to=message.message_id, menu=ReplyKeyboardRemove(selective=True))
 
         items = self._api_client.get_approvals()
+
+        exact_matches = list(filter(lambda x: x["id"] == identifier, items))
+        if len(exact_matches) > 0:
+            execute(update, context, exact_matches[0], {})
+            return
+
         self._response_handler.await_user_selection(
             update, context, identifier, choices=items, key=lambda x: x["identifier"],
             callback=execute,
@@ -258,6 +295,8 @@ class KeelTelegramBot:
         Handles incoming notifications (via Webhook)
         :param data: notification data
         """
+        KEEL_NOTIFICATION_COUNTER.inc()
+
         identifier = data.get("identifier", None)
         title = data.get("name", None)
         type = data.get("type", None)
@@ -392,10 +431,12 @@ class KeelTelegramBot:
                 self._api_client.approve(approval_id, approval_identifier, from_user.full_name)
                 answer_text = f"Approved '{approval_identifier}'"
                 keyboard = self._build_inline_keyboard({"Approved": BUTTON_DATA_NOTHING})
+                KEEL_APPROVAL_ACTION_COUNTER.labels(action="approve", identifier=approval_identifier).inc()
             elif data == BUTTON_DATA_REJECT:
                 self._api_client.reject(approval_id, approval_identifier, from_user.full_name)
                 answer_text = f"Rejected '{approval_identifier}'"
                 keyboard = self._build_inline_keyboard({"Rejected": BUTTON_DATA_NOTHING})
+                KEEL_APPROVAL_ACTION_COUNTER.labels(action="reject", identifier=approval_identifier).inc()
             else:
                 bot.answer_callback_query(query_id, text="Unknown button")
                 return
